package com.link.core.basic.es.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.alibaba.fastjson.parser.Feature;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import com.link.core.basic.es.config.ElasticsearchConfig;
import com.link.core.basic.service.BasicServiceException;
import com.link.core.objeccache.RedisCache;
import com.link.core.util.BeanTools;
import com.link.core.util.RedisGenUtilAbstract;
import com.link.core.util.RedisUtil;
import com.link.core.util.StringUtils;
import com.link.core.util.redisclient.LinkRedisClient;
import java.time.Duration;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import javax.annotation.Resource;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.client.indices.GetMappingsRequest;
import org.elasticsearch.client.indices.GetMappingsResponse;
import org.elasticsearch.cluster.metadata.MappingMetadata;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

@Component
public class EsCacheUtils {
   private static final Logger logger = LoggerFactory.getLogger(EsCacheUtils.class);
   @Resource
   private RestHighLevelClient restHighLevelClient;
   @Resource
   private ElasticsearchConfig elasticsearchConfig;
   private static LoadingCache<String, Map<String, Object>> esConfigCache = CacheBuilder.newBuilder().initialCapacity(50).maximumSize(100L).expireAfterWrite(Duration.ofDays(1L)).build(new CacheLoader<String, Map<String, Object>>() {
      public Map<String, Object> load(String key) {
         LinkRedisClient jedis = null;

         Map var3;
         try {
            jedis = RedisUtil.getJedis();
            var3 = (Map)JSONObject.parseObject(jedis.hget("es_config_index_sql", key), new TypeReference<Map<String, Object>>() {
            }, new Feature[0]);
         } finally {
            RedisUtil.returnResource(jedis);
         }

         return var3;
      }
   });
   private static LoadingCache<String, List<Map<String, Object>>> esSubConfigCache = CacheBuilder.newBuilder().initialCapacity(50).maximumSize(100L).expireAfterWrite(Duration.ofDays(1L)).build(new CacheLoader<String, List<Map<String, Object>>>() {
      public List<Map<String, Object>> load(String key) {
         LinkRedisClient jedis = null;

         try {
            jedis = RedisUtil.getJedis();
            String subConfigStr = jedis.hget("es_sub_config_index_sql", key);
            if (StringUtils.isBlank(subConfigStr)) {
               ArrayList var11 = new ArrayList();
               return var11;
            } else {
               JSONArray objects = JSONArray.parseArray(subConfigStr);
               List<Map<String, Object>> list = new ArrayList(objects.size());

               for(int i = 0; i < objects.size(); ++i) {
                  Map<String, Object> map = (Map)JSONObject.parseObject(JSONObject.toJSONString(objects.get(i)), new TypeReference<Map<String, Object>>() {
                  }, new Feature[0]);
                  list.add(map);
               }

               ArrayList var12 = list;
               return var12;
            }
         } finally {
            RedisUtil.returnResource(jedis);
         }
      }
   });

   public JSONObject getIndexMapping(String index) throws Exception {
      String redisKey = RedisGenUtilAbstract.genRedisKey("es_indexmapping:%s", new String[]{index});

      String jsonStr;
      try {
         LinkRedisClient redisClient = RedisUtil.getJedis();
         jsonStr = redisClient.get(redisKey);
         if (StringUtils.isBlank(jsonStr)) {
            return this.refreshIndexMapping(index);
         }
      } catch (Exception var6) {
         Exception e = var6;
         throw new BasicServiceException("从缓存中获取ES索引的mapping关系失败：" + e.getMessage());
      }

      return JSONObject.parseObject(jsonStr);
   }

   public JSONObject refreshIndexMapping(String index) throws Exception {
      GetMappingsRequest request = new GetMappingsRequest();
      request.indices(new String[]{index});
      GetMappingsResponse getMappingResponse = this.restHighLevelClient.indices().getMapping(request, RequestOptions.DEFAULT);
      MappingMetadata indexMapping = (MappingMetadata)getMappingResponse.mappings().get(index);
      if (indexMapping == null) {
         throw new BasicServiceException("当前索引在ES没有配置字段映射信息:" + index);
      } else {
         Map<String, Object> resultMap = indexMapping.sourceAsMap();
         JSONObject json = JSONObject.parseObject(JSON.toJSONString(resultMap));
         JSONObject resultJson = json.getJSONObject("properties");
         if (resultJson.isEmpty()) {
            throw new BasicServiceException("当前索引配置字段为空，请检查ES数据 : " + index);
         } else {
            LinkRedisClient redisClient = null;

            try {
               String redisKey = RedisGenUtilAbstract.genRedisKey("es_indexmapping:%s", new String[]{index});
               redisClient = RedisUtil.getJedis();
               redisClient.setex(redisKey, RedisCache.getCacheTime(), resultJson.toJSONString());
               return resultJson;
            } catch (Exception var10) {
               Exception e = var10;
               throw new BasicServiceException("刷新ES索引的Mapping缓存失败：" + e.getMessage());
            }
         }
      }
   }

   public String getFieldType(String index, String... fields) throws Exception {
      if (null != fields && fields.length > 0) {
         if (fields[0].split("\\.").length > 1) {
            fields = fields[0].split("\\.");
         }

         JSONObject jsonObject = this.getIndexMapping(index);
         JSONObject fieldObject = jsonObject.getJSONObject(fields[0]);
         if (null == fieldObject) {
            throw new BasicServiceException("获取索引：" + index + "中的字段：" + fields[0] + "失败");
         } else {
            if (fields.length > 1) {
               fieldObject = fieldObject.getJSONObject("properties").getJSONObject(fields[1]);
               if (null == fieldObject) {
                  throw new BasicServiceException("获取索引：" + index + "中的子对象[" + fields[0] + "]字段：" + fields[1] + "失败");
               }
            }

            return fieldObject.getString("type");
         }
      } else {
         throw new BasicServiceException("传入的字段为空");
      }
   }

   public JSONObject getConfigByIndexAlias(String indexAlias) throws Exception {
      String redisKey = RedisGenUtilAbstract.genRedisKey("es_config:%s", new String[]{indexAlias});

      String jsonStr;
      try {
         LinkRedisClient redisClient = RedisUtil.getJedis();
         jsonStr = redisClient.get(redisKey);
         if (StringUtils.isBlank(jsonStr)) {
            return this.refreshConfigByIndexAlias(indexAlias);
         }
      } catch (Exception var6) {
         Exception e = var6;
         throw new BasicServiceException("从缓存中获取数据转换配置的信息失败：" + e.getMessage());
      }

      return JSONObject.parseObject(jsonStr);
   }

   public JSONObject refreshConfigByIndexAlias(String indexAlias) throws Exception {
      new JSONObject();
      String schema = this.elasticsearchConfig.getConfigSchema();
      if (StringUtils.isBlank(schema)) {
         schema = "linkbase";
      }

      String sql = "SELECT * FROM " + schema + ".lnk_transfer_config WHERE is_effective =  'ENABLE' and index_nicname = '" + indexAlias + "' limit 1  ";
      JdbcTemplate jdbcTemplate = (JdbcTemplate)BeanTools.getBean(JdbcTemplate.class);

      JSONObject config;
      try {
         assert jdbcTemplate != null;

         List<Map<String, Object>> resultList = jdbcTemplate.queryForList(sql);
         if (CollectionUtils.isEmpty(resultList)) {
            logger.error("当前索引配置中不存在有效的配置信息，索引别名：" + indexAlias);
            throw new BasicServiceException("不存在有效的索引配置");
         }

         if (resultList.size() > 1) {
            logger.error("当前索引配置中存在多条有效的配置信息，索引别名：" + indexAlias);
            throw new BasicServiceException("存在多条有效的索引配置");
         }

         config = new JSONObject((Map)resultList.get(0));
      } catch (Exception var9) {
         Exception e = var9;
         logger.error("查询索引别名：" + indexAlias + "的数据转换配置出错：" + e.getMessage());
         throw new BasicServiceException(e.getMessage());
      }

      try {
         String redisKey = RedisGenUtilAbstract.genRedisKey("es_config:%s", new String[]{indexAlias});
         LinkRedisClient redisClient = RedisUtil.getJedis();
         redisClient.setex(redisKey, RedisCache.getCacheTime(), config.toJSONString());
         return config;
      } catch (Exception var8) {
         Exception e = var8;
         throw new BasicServiceException("刷新ES数据转换配置缓存失败：" + e.getMessage());
      }
   }

   public void refresh(String index, String indexAlias) throws Exception {
      this.refreshIndexMapping(index);
      this.refreshConfigByIndexAlias(indexAlias);
   }

   public static void refreshEsConfigCache(String key) {
      esConfigCache.refresh(key);
   }

   public static Map<String, Object> getEsConfigCache(String key) throws Exception {
      return (Map)esConfigCache.get(key);
   }

   public static void refreshEsSubConfigCache(String key) {
      esSubConfigCache.refresh(key);
   }

   public static List<Map<String, Object>> getEsSubConfigCache(String key) throws Exception {
      return (List)esSubConfigCache.get(key);
   }
}
